<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Jobs.php 2894 2012-08-01 14:23:23Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for Jobs table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_Jobs extends Giscuit_Db_Table_Abstract
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'jobs';

    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'id';

    /**
     * Get items
     * 
     * @param array $options Available options:
     * <ul><li>limit integer - Number of results to get</li>
     * <li>offset integer - Offset number</li>
     * <li>whereStatus string - Fetch jobs with these statuses</li>
     * <li>allActiveJobs boolean - Fetch all active jobs</li>
     * <li>order string - Sort column name and direction</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'id' , 
            'title' ,
            'desc' , 
            'contact_address' , 
            'contact_name' , 
            'contact_phone' , 
            'priority' , 
            'date' => "extract(epoch from date)" , 
            'deadline' => "extract(epoch from deadline)" ,
            'status',
            'workflow_id',
            'notify',
            'to_user_id',
            'from_user_id',
            'the_geom' => 'ST_AsText(the_geom)'
        );
        
        $select->from($this, $fields);
        
        if (isset($options['whereId'])) {
            $select->where($this->_name . '.id = ?', $options['whereId']);
        }
        
        if (isset($options['whereCurrentUser'])) {
            $select->where($this->_name . '.to_user_id = ?', Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct());
        }
        
        if(isset($options['whereStatus'])) {
            $select->where($this->_name . '.status IN ' . $options['whereStatus']);
        }
        
        if(isset($options['limit'])) {
            $select->limit($options['limit'], $options['offset']);
        }
        
        if(isset($options['order'])) {
            $select->order($options['order']);
        }
        
        if (! empty($options['withActionsWorkflows'])) {
            $select->joinLeft(array(
                $this->_schema . '.jobs_actions_workflows'
            ), "jobs_actions_workflows.id = {$this->_name}.workflow_id", 'name_id');
            
            $this->joinMultilangField($select, 'name', 'jobs_actions_workflows');
        }
        
        $result = $this->fetchAll($select)->toArray();
        
        $result = $this->withActions($select, $result, $options);
        
        return $result;
    }
    
    /**
     * Update status
     * 
     * @param integer $id The id of the job
     * @param string $status New status
     * @return void
     */
    public function updateStatus ($id, $status)
    {
        $rows = $this->find($id);
        $row = $rows->current();
        
        $row->status = $status;
        
        //Save
        $row->save();
    }
    
    /**
     * Get total number of jobs
     * 
     * @param array $options Available options:
     * <ul><li>whereStatus string - Fetch jobs with these statuses</li></ul>
     * @return Zend_Db_Table_Row
     */
    public function getTotal (array $options = array())
    {
        $select = $this->select();
        $select->from($this, array('total' => new Zend_Db_Expr('COUNT(*)')));
        
        if(isset($options['whereStatus'])) {
            $select->where($this->_name . '.status IN ' . $options['whereStatus']);
        }
        
        return $this->fetchRow($select);
    }
    
    /**
     * Add item
     * 
     * @param Zend_Form $data A form object populated with request data
     * @param array $geometry Array with x and y coordinates of a point
     * @return integer The id of the new element
     */
    public function addItem ($data, $geometry)
    {
        $config = Zend_Registry::get('config');
        $newRow = $this->createRow();
        
        $newRow->to_user_id = $data->getValue('to_user');
        $newRow->from_user_id = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        $newRow->title = $data->getValue('title');
        $newRow->desc = $data->getValue('desc');
        $newRow->contact_address = $data->getValue('contact_address');
        $newRow->contact_name = $data->getValue('contact_name');
        $newRow->contact_phone = $data->getValue('contact_phone');
        $newRow->priority = $data->getValue('priority');
        $newRow->date = new Zend_Db_Expr('now()');
        $newRow->deadline = $data->getValue('deadline') ? $data->getValue('deadline') : NULL;
        $newRow->status = 'pending';
        $newRow->workflow_id = $data->getValue('workflow');
        $newRow->notify = $data->getValue('notify');
        $newRow->the_geom = new Zend_Db_Expr("ST_MakePoint({$geometry['x']}, {$geometry['y']})");
        
        return $newRow->save();
    }
    
    /**
     * Edit item
     * 
     * @param intreger $id The id of the row to edit 
     * @param Zend_Form $data A form object populated with request data
     * @return void
     */
    public function editItem ($id, $data)
    {
        $row = $this->find($id)->current();
        
        $row->to_user_id = $data->getValue('to_user');
        $row->title = $data->getValue('title');
        $row->desc = $data->getValue('desc');
        $row->contact_address = $data->getValue('contact_address');
        $row->contact_name = $data->getValue('contact_name');
        $row->contact_phone = $data->getValue('contact_phone');
        $row->priority = $data->getValue('priority');
        $row->deadline = $data->getValue('deadline');
        $row->workflow_id = $data->getValue('workflow');
        $row->notify = $data->getValue('notify');
        
        if ($data->getValue('revert') == 't') {
            $row->status = 'pending';
        }
        
        $row->save();
    }
    
    /**
     * Fetch actions data
     * 
     * @param Zend_Db_Select $select
     * @param array $result
     * @param array $options Available options:
     * <ul><li>withActions bool - Fetch actions data</li></ul>
     * @return array Parsed result
     */
    public function withActions ($select, $result, array $options = array())
    {
        if (empty($options['withActions'])) {
            return $result;
        }
        
        $jobActionsTable = new Giscuit_Db_Table_JobsActions();
        
        //Get job and workflow ids
        $jobIds = array();
        foreach ($result as $job) {
            $jobIds[] = $job['id'];
        }
        
        //Get job actions
        if (! empty($result)) {
            //Get job actions
            $jobIds = '(' . implode(',', $jobIds) . ')';
            $jobActions = $jobActionsTable->getItems(array('jobIds' => $jobIds));
        }
        
        foreach ($result as $key => $value) {
            //Set action history
            $actions = array();
            foreach ($jobActions as $action) {
                if ($action['job_id'] == $value['id']) {
                    $actions[] = array(
                        'action' => $action['action_map_id'],
                        'comment' => $action['comment'],
                        'date' => $action['date'],
                        'the_geom' => $action['the_geom']
                    );
                }
            }
            $result[$key]['actions'] = $actions;
        }
        
        return $result;
    }
}
