<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 *
 *
 * @version    $Id: Articles.php 1629 2011-08-17 16:11:48Z vlad.bruma $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 *
 */

/**
 * @see Giscuit_Db_Table_Multilang
 */
require_once 'Giscuit/Db/Table/Multilang.php';

/**
 * Class for incidents table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 *
 */
class Giscuit_Db_Table_Layer_Incidents extends Giscuit_Db_Table_Multilang
{
    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'gid';

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'incidents';
    
    /**
     * Names of multilang fields inside the table
     * 
     * @var array
     */
    protected $_multilangTextColumns = array('category');

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct ($options = array())
    {
        $config = Zend_Registry::get('config');

        $this->_schema = $config->database->schema . '_layers';
        $this->_mainSchema = $config->database->schema;
                
        parent::__construct($options);
    }
    
    /**
     * Get items
     *   
     * @param array $options <ul>Available options:
     * <li>limit integer - Maximum number of rows to return</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array (
            'gid',
            'active',
            'title', 
            'description', 
            'date' => "to_char({$this->_name}.date, '" . GT_DATE_STYLE_PHP . "')", 
            'fname', 
            'lname', 
            'email', 
            'the_geom' => new Zend_Db_Expr('ST_AsText(' . $this->getAdapter()->quoteIdentifier($this->_name . '.the_geom') . ')')
        );
        
        if (isset($options['withCentroid'])) {
            $fields['centroid'] = new Zend_Db_Expr('ST_AsText(ST_Centroid(' . $this->getAdapter()->quoteIdentifier($this->_name . '.the_geom') . '))');
        }
        
        $select->from($this, $fields);
        
        $select->joinLeft(array(
            'files' => $this->_mainSchema . '.files' ,
            ), "{$this->_name}.image = files.id" ,
            array('image_id' => 'id', 'image_name' => 'name')
        );
        
        $select->joinLeft(array(
            'category' => $this->_schema . '.incidents_category' ,
            ), "{$this->_name}.category_id = category.id" ,
            array('category_color' => 'color')
        );
        
        if (isset($options['whereGid'])) {
            $select->where($this->_name . '.gid = ?', $options['whereGid']);
        }
        
        if (isset($options['whereActive'])) {
            $select->where($this->_name . '.active = ?', $options['whereActive']);
        }
        
        if (isset($options['whereCategory']) && $options['whereCategory'] != 0) {
            $select->where($this->_name . '.category_id = ?', $options['whereCategory']);
        }
        
        $this->joinMultilangField($select, 'name', 'category', $this->_mainSchema);

        //Set the result limit from the plugin's parameters
        if (isset($options['limit'])) {
            $select->limit($options['limit']);
        }
        
        $select->order('gid DESC');
                
        //Fetch the results from the database
        $result = $this->fetchAll($select);
        
        return $result;
    }

    /**
     * Add item
     * 
     * @param Zend_Form $data A form object populated with request data
     * @return integer The id of the new element
     */
    public function addItem ($data)
    {
        $config = Zend_Registry::get('config');
        
        $newRow = $this->createRow();
        
        $newRow->title = $data->getValue('title');
        $newRow->description = $data->getValue('description');
        $newRow->date = new Zend_Db_Expr('now()');
        $newRow->category_id = $data->getValue('category_id');
        $newRow->fname = $data->getValue('fname');
        $newRow->lname = $data->getValue('lname');
        $newRow->email = $data->getValue('email');

        //Save file
        if ($data->getElement('image') != NULL && $data->getElement('image')->receive() && 
            $data->getElement('image')->getValue() != NULL) {
            $info = $data->getElement('image')->getFileInfo();
            $fileData = array(
                'type' => $info['image']['type'] , 
                'file' => $info['image']['destination'] . DIR_SEP . $info['image']['name'],
                'name' => htmlspecialchars($info['image']['name'], ENT_QUOTES)
            );
    
            $filesTable = new Giscuit_Db_Table_Files();
    
            $id = $filesTable->addItem($fileData);
                        
            $newRow->image = $id;
        }

        $newRow->the_geom = new Zend_Db_Expr("ST_GeomFromText('" . $data->getValue('the_geom') . "', {$config->map->srid})");

        return $newRow->save();
    }

    /**
     * Edit item
     * 
     * @param integer $gid The id of the row to edit 
     * @return integer The gid of the element
     */
    public function editStatus ($gid)
    {
        $rows = $this->find($gid);

        if ($rows->count() == 0) {
            throw new Giscuit_Exception("Invalid incident id :" . $gid);
        }
        
        $row = $rows->current();
        
        if ($row->active == 't') {
            $row->active = 'f';
        } else {
            $row->active = 't';
        }
        
        return $row->save();
    }
}