<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Multilang.php 3366 2013-04-12 12:54:39Z vlad.bruma $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Abstract class for a multilang table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
abstract class Giscuit_Db_Table_Multilang extends Giscuit_Db_Table_Abstract
{
    /**
     * Contains the names of multilang fields inside the table
     * 
     * @var array
     */
    protected $_multilangTextColumns = array(
    );
    
    /**
     * Class wide constant for INSERT
     */
    const INSERT = 1;

    /**
     * Class wide constant for UPDATE
     */
    const UPDATE = 2;

    /**
     * Insert or update text in a multilanguage table
     * 
     * @param array $arr Fields with multilang field name and value, example: arr[text] = value 
     * @param Zend_Db_Table_Row $row Inserted or updated row 
     * @param mixed $action By default insert the multilang text or pass the Giscuit_Db_Table_Multilang::UPDATE to update
     * @return void
     */
    public function manageMultilangText (array $arr, $row, $action = self::INSERT)
    {
           
        $languages = array_keys(Zend_Registry::get('config')->language->toArray());
        
        $textTable = new Giscuit_Db_Table_Text();
        
        foreach ($arr as $fieldKey => $field) {
            
            $fieldName = $fieldKey . '_id';
            
            if ($action == self::INSERT) {
                
                //Create new row
                $textRow = $textTable->createRow();
            
            } elseif ($action == self::UPDATE) {
                
                //Find row
                $textRow = $textTable->find($row->$fieldName);
                $textRow = $textRow->current();
            
            }
            
            //Update text fields
            foreach ($field as $langKey => $text) {
                if (in_array($langKey, $languages)) {
                    $textRow->$langKey = $text;
                }
            }
            
            if ($action == self::INSERT) {
                
                $fieldId = $textRow->save();
                $row->$fieldName = $fieldId;
            
            } elseif ($action == self::UPDATE) {
                $textRow->save();
            }
        }
    
    }

    /**
     * Add all the tables multilanguage fields to the select object
     * 
     * @param Zend_Db_Select $select
     * @return void
     */
    public function addMultilangFields ($select)
    {
        $fields = $this->_multilangTextColumns;
        
        $languages = Zend_Registry::get('config')->language->toArray();
        
        $select->setIntegrityCheck(false);
        
        foreach ($fields as $field) {
            $arr = array();
            
            foreach ($languages as $key => $language) {
                $arr[$field . '_' . $key] = $key;
            }
            
            $select->joinLeft(array(
                'text_' . $field => $this->_schema . '.text'
            ), "text_{$field}.id = {$this->_name}.{$field}_id", $arr);
        }
    
    }
    
    /**
     * Check for multilanguage fields addition
     * 
     * @param Zend_Db_Select $select
     * @param array $options <ul>Available options:
     * <li>withMultilangFields bool - Add all the tables multilanguage fields to the select object</li></ul>
     * @return void
     */
    public function withMultilangFields($select, array $options = array())
    {
        if (empty($options['withMultilangFields'])) {
            return;    
        }
        
        $this->addMultilangFields($select);
    }
    
    /**
     * Deletes existing rows.
     *
     * @param  array|string $where SQL WHERE clause(s)
     * @return int          The number of rows deleted
     */
    public function delete($where)
    {
        $tableSpec = ($this->_schema ? $this->_schema . '.' : '') . $this->_name;
        
        $select = $this->select();
        $select->setIntegrityCheck(false);

        //Get text ids 
        $cols = array();
        foreach($this->_multilangTextColumns as $value) {
            $cols[] = $value . '_id';
        }
        $select->from($this, $cols);
        $select->where($where);
        $result = $this->fetchAll($select);
        
        //Create text delete query
        $textIdsArr = array();
        foreach($result->toArray() as $row) {
            foreach ($cols as $col) {
                $textIdsArr[] = $row[$col];
            }
        }
        $textIds = 'id = ' . implode(' OR id = ', $textIdsArr);
        
        //Delete 
        $delResult = $this->_db->delete($tableSpec, $where);
        
        //Delete text ids
        $textTable = new Giscuit_Db_Table_Text();
        
        if(! empty($textIds)) {
            $textTable->delete($textIds);
        }
        
        return $delResult;
        
    }
    
    /**
     * Get all the tables multilanguage fields and return the data as an array
     * 
     * @param array $values Multilanguage fields
     * @return array
     */
    public function getMultilangPostData ($values)
    {
        $result = array();
        
        $fields = $this->_multilangTextColumns;
        
        $languages = Zend_Registry::get('config')->language->toArray();
        
        foreach ($languages as $key => $language) {
            foreach ($fields as $field) {
                $result[$field][$key] = $values[$field . '_' . $key];
            }
        }
        
        return $result;
    
    }

}



