<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Styles.php 3379 2013-04-24 12:16:19Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for classes table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_Styles extends Giscuit_Db_Table_Abstract
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'styles';

    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'id';

    /**
     * Get styles
     * 
     * @param array $options Available options:
     * <ul><li>whereStm bool - Fetch rows that match the given statement</li>
     * <li>whereClassId integer - Fetch rows that match the given class id</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'id' , 
            'class_id' , 
            'color' , 
            'width' , 
            'outlinecolor' , 
            'size' , 
            'symbolname' , 
            'opacity' ,
            'angle' , 
            'backgroundcolor' ,
            'pattern' ,
            'position' ,
            'gap' ,
            'linecap' ,
            'linejoin' ,
            'linejoinmaxsize'
        );
        
        $select->from($this, $fields);
        
        if (isset($options['whereStm'])) {
            $select->where($options['whereStm']);
        }
        
        if (isset($options['whereClassId'])) {
            $select->where($this->_name . '.class_id = ?', $options['whereClassId']);
        }
        
        $this->whereId($select, $options);
        
        $select->order($this->_name . '.order asc');
        
        $result = $this->fetchAll($select);
        
        return $result;
    }

    /**
     * Update item
     * 
     * @param integer $id The id of the row to edit 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * @return integer The id of the element
     */
    public function editItem ($id, $data)
    {
        
        $rows = $this->find($id);
        $row = $rows->current();
        
        //Get column
        $column = $data->column;
        
        if(empty($data->data)) {
            $data->data = NULL;
        }
        
        $row->$column = $data->data;
        
        Giscuit_Map_Map::clearLegend();
        
        return $row->save();
    
    }

    /**
     * Add item
     * 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * @param array $options 
     * @return Zend_Db_Table_Row
     */
    public function addItem ($data, array $options = array())
    {
        $newRow = $this->createRow();
        $newRow->class_id = $data->class_id;
        
        $pkey = $newRow->save();
        
        $newRow->order = $pkey;
        $newRow->save();
        
        return $newRow;
    }
    
    /**
     * Add initial styles to the given class and type
     * 
     * @param integer $classId Id of the class
     * @param string $type Geometry type
     * @return Zend_Db_Table_Row
     */
    public function addInitial($classId, $type)
    {
        $newRow = $this->createRow();
        $newRow->class_id = $classId;

        $newRow->width = 1;
        $newRow->color = Giscuit_Map_Layer_Vector::rgb2Hex(rand(0, 255), rand(0, 255), rand(0, 255));

        if($type == 'Polygon') {
            $newRow->outlinecolor = Giscuit_Map_Layer_Vector::rgb2Hex(rand(0, 255), rand(0, 255), rand(0, 255));
        } elseif($type == 'Line') {
            $newRow->width = 2;
        } elseif($type == 'Point') {
            $newRow->size = 12;
            $newRow->symbolname = 'circle';
            $newRow->outlinecolor = Giscuit_Map_Layer_Vector::rgb2Hex(rand(0, 255), rand(0, 255), rand(0, 255));
        }
        
        $pkey = $newRow->save();
                
        $newRow->order = $pkey;
        $newRow->save();
        
        return $newRow;
    }

    /**
     * Add classify
     * 
     * @param Zend_Filter_Input $input A Zend_Filter_Input object populated with request data
     * @param integer $classId Id of the class
     * @param string $color Style color
     * @return void
     */
    public function addClassify ($input, $classId, $color)
    {
        $newRow = $this->createRow();
        $newRow->class_id = $classId;
        $newRow->color = $color;
        $newRow->width = $input->classify_width;
        $newRow->outlinecolor = $input->classify_outlinecolor;
        $newRow->size = $input->classify_size;
        $newRow->symbolname = $input->classify_symbolname;
        $newRow->opacity = $input->classify_opacity;
        $newRow->backgroundcolor = $input->classify_backgroundcolor;
        $newRow->angle = $input->classify_angle;
        
        $pkey = $newRow->save();
        
        $newRow->order = $pkey;
        $newRow->save();
    }
    
    /**
     * Check if style belongs to current user
     * 
     * @param integer $id Style id
     * @return void
     */
    public function checkCurrUserOwnStyle($id)
    {
        $rows = $this->getItems(array(
            'whereId' => $id
        ))->toArray();
        
        if (count($rows) == 0) {
            throw new Exception("Invalid style id ({$id})");
        }
        
        $classesTable = new Giscuit_Db_Table_Classes();
        $classesTable->checkCurrUserOwnClass($rows[0]['class_id']);
    }
}
